# ../addons/source-python/plugins/parachute/parachute.py


# ============================================================================
# >> IMPORTS
# ============================================================================
# Source.Python Imports
from cvar_c import FCVAR_NOTIFY
from constants_c import InButtons
from constants_c import MoveTypes
from constants_c import PlayerFlags
from mathlib_c import Vector
#   Config
from config.manager import ConfigManager
#   ConVars
from cvars import ServerVar
#   Events
from events import Event
#   Filters
from filters.players import PlayerIter
#   Messages
from messages import HintText
#   Players
from players.entity import PlayerEntity
from players.helpers import edict_from_playerinfo
#   Plugins
from plugins.info import PluginInfo
#   Tick
from tick import Tick
from tick.delays import TickDelays
#   Translations
from translations.strings import LangStrings


# ============================================================================
# >> INFORMATIONS
# ============================================================================
informations = PluginInfo()
informations.author = 'L\'In20Cible'
informations.basename = __name__.rsplit('.')[~0]
informations.description = 'You know...'
informations.name = informations.basename.title()
informations.version = '0.01'
informations.url = 'http://www.sourcepython.com/index.php'

ServerVar('parachute_version', informations.version, FCVAR_NOTIFY,
    informations.description)
    
    
# ============================================================================
# >> GLOBAL VARIABLES
# ============================================================================
# Create and execute the configuration file...
configuration = ConfigManager(informations.basename)
parachute_advert = configuration.cvar('parachute_advert', '1',
    description='Enable/Disable the advert every round start.')
parachute_button = configuration.cvar('parachute_button', 'SPEED',
    description='Defines the button to use the parachute.')
parachute_falling_speed = configuration.cvar('parachute_falling_speed', '10',
    description='Defines the falling speed of the parachute.')
configuration.write()
configuration.execute()

# Parse the translations files...
translations = LangStrings(informations.basename)

# Get a global HintText...
advert = HintText(message=translations['Advert'])


# ============================================================================
# >> LISTENER CALLBACKS
# ============================================================================
@Tick
def tick_listener():
    '''Fired each game frame...'''
    
    # Loop through all living, human players...
    for playerinfo, edict in PlayerIter('alive', 'bot', ['info', 'edict']):
        
        # Get the player falling velocity...
        velocity = edict.get_prop_float('m_Local.m_flFallVelocity')
        
        # Is the player not falling?
        if (velocity < 1.0 or
        
        # Is the player not holding his parachute key?
        not playerinfo.get_last_user_command().buttons & getattr(InButtons,
            parachute_button.get_string().upper()) or
            
        # Is the player currently in a ladder?
        edict.get_prop_int('movetype') & MoveTypes.LADDER or
        
        # Is the player currently in water?
        edict.get_prop_int('m_fFlags') & PlayerFlags.INWATER):
            
            # If any of the check above was True, no need to go further...
            continue
            
        # Revert the falling velocity to slow down the player speed...
        edict.set_prop_vector('localdata.m_vecBaseVelocity',
            Vector(0, 0, velocity + (
                parachute_falling_speed.get_float() * -1)))
                
                
# ============================================================================
# >> GAME EVENTS
# ============================================================================
@Event
def round_start(game_event):
    '''Fired at the beginning of every round...'''
    
    # Is the advert disabled?
    if not parachute_advert.get_bool():
        
        # No need to go further...
        return
        
    # Send the advert...
    # NOTE: Since ResetHud is sent every round, we need to wait a bit before
    #   sending a HintText message...
    TickDelays.delay(0.5, advert.send,
        button=parachute_button.get_string().lower())